@@ -23,19 +23,21 @@
 #include "settings.h"
 #include "skinlistdelegate.h"
 
-#include <KApplication>
-#include <KFileDialog>
+#include <KIO/CopyJob>
 #include <KIO/DeleteJob>
-#include <KIO/NetAccess>
+#include <KJobUiDelegate>
+#include <KLocalizedString>
 #include <KMessageBox>
-#include <KStandardDirs>
 #include <KTar>
-#include <KUrl>
 
-#include <KNS3/DownloadDialog>
-#include <knewstuff3/downloadmanager.h>
+#include <downloaddialog.h>
+#include <downloadmanager.h>
 
+#include <QDir>
+#include <QDirIterator>
 #include <QFile>
+#include <QFileDialog>
+#include <QPointer>
 #include <QStandardItemModel>
 
 #include <unistd.h>
@@ -61,22 +63,22 @@ AppearanceSettings::AppearanceSettings(QWidget* parent) : QWidget(parent)
     connect(installButton, SIGNAL(clicked()), this, SLOT(installSkin()));
     connect(removeButton, SIGNAL(clicked()), this, SLOT(removeSelectedSkin()));
 
-    installButton->setIcon(KIcon("folder"));
-    removeButton->setIcon(KIcon("edit-delete"));
-    ghnsButton->setIcon(KIcon("get-hot-new-stuff"));
+    installButton->setIcon(QIcon(QStringLiteral("folder")));
+    removeButton->setIcon(QIcon(QStringLiteral("edit-delete")));
+    ghnsButton->setIcon(QIcon(QStringLiteral("get-hot-new-stuff")));
 
     m_knsConfigFileName = QLatin1String("yakuake.knsrc");
-    m_knsDownloadManager = new KNS3::DownloadManager(m_knsConfigFileName);
+    m_knsDownloadManager = new KNSCore::DownloadManager(m_knsConfigFileName);
 
-    connect(ghnsButton, SIGNAL(clicked()), this, SLOT(getNewSkins()));
+    connect(ghnsButton, &QPushButton::clicked, this, &AppearanceSettings::getNewSkins);
 
     m_selectedSkinId = Settings::skin();
 
     // Get all local skin directories.
     // One for manually installed skins, one for skins installed
     // through KNS3.
-    m_localSkinsDir = KStandardDirs::locateLocal("data", "yakuake/skins/");
-    m_knsSkinDir = KStandardDirs::locateLocal("data", "yakuake/kns_skins/");
+    m_localSkinsDir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QStringLiteral("/yakuake/skins/");
+    m_knsSkinDir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QStringLiteral("/yakuake/kns_skins/");
 
     populateSkinList();
 }
@@ -99,44 +101,13 @@ void AppearanceSettings::populateSkinList()
 {
     m_skins->clear();
 
-    QStringList titleDirs = KGlobal::dirs()->findAllResources("data", "yakuake/skins/*/title.skin")
-        + KGlobal::dirs()->findAllResources("data", m_knsSkinDir + "*/title.skin");
-    QStringList tabDirs = KGlobal::dirs()->findAllResources("data", "yakuake/skins/*/tabs.skin")
-        + KGlobal::dirs()->findAllResources("data", m_knsSkinDir + "*/tabs.skin");
+    QStringList allSkinLocations;
+    allSkinLocations << QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("/yakuake/skins/"), QStandardPaths::LocateDirectory);
+    allSkinLocations << QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("/yakuake/kns_skins/"), QStandardPaths::LocateDirectory);
 
-    QStringList skinDirs;
-    QStringListIterator i(titleDirs);
-
-    while (i.hasNext())
+    Q_FOREACH (const QString& skinLocation, allSkinLocations)
     {
-        const QString& titleDir = i.next();
-        const QString& skinDir(titleDir.section('/', 0, -2));
-
-        if (tabDirs.contains(skinDir + "/tabs.skin")
-            && !skinDirs.filter(QRegExp(QRegExp::escape(skinDir.section('/', -2, -1)) + '$')).count())
-        {
-            skinDirs << skinDir;
-        }
-    }
-
-    if (skinDirs.count() == 0)
-        return;
-
-    QStringListIterator i2(skinDirs);
-
-    while (i2.hasNext())
-    {
-        const QString& skinDir = i2.next();
-
-        QStandardItem* skin = createSkinItem(skinDir);
-
-        if (!skin)
-            continue;
-
-        m_skins->appendRow(skin);
-
-        if (skin->data(SkinId).toString() == m_selectedSkinId)
-            skinList->setCurrentIndex(skin->index());
+        populateSkinList(skinLocation);
     }
 
     m_skins->sort(0);
@@ -144,9 +115,32 @@ void AppearanceSettings::populateSkinList()
     updateRemoveSkinButton();
 }
 
+void AppearanceSettings::populateSkinList(const QString& installLocation)
+{
+    QDirIterator it(installLocation, QDir::Dirs | QDir::NoDotAndDotDot);
+
+    while (it.hasNext())
+    {
+        const QDir& skinDir(it.next());
+
+        if (skinDir.exists(QStringLiteral("title.skin")) && skinDir.exists(QStringLiteral("tabs.skin")))
+        {
+            QStandardItem* skin = createSkinItem(skinDir.absolutePath());
+
+            if (!skin)
+                continue;
+
+            m_skins->appendRow(skin);
+
+            if (skin->data(SkinId).toString() == m_selectedSkinId)
+                skinList->setCurrentIndex(skin->index());
+        }
+    }
+}
+
 QStandardItem* AppearanceSettings::createSkinItem(const QString& skinDir)
 {
-    QString skinId = skinDir.section('/', -1, -1);
+    QString skinId = skinDir.section(QStringLiteral("/"), -1, -1);
     QString titleName, tabName, skinName;
     QString titleAuthor, tabAuthor, skinAuthor;
     QString titleIcon, tabIcon;
@@ -156,10 +150,10 @@ QStandardItem* AppearanceSettings::createSkinItem(const QString& skinDir)
     // KNS3 skins are found in.
     bool isKnsSkin = skinDir.startsWith(m_knsSkinDir);
 
-    KConfig titleConfig(skinDir + "/title.skin", KConfig::SimpleConfig);
+    KConfig titleConfig(skinDir + QStringLiteral("/title.skin"), KConfig::SimpleConfig);
     KConfigGroup titleDescription = titleConfig.group("Description");
 
-    KConfig tabConfig(skinDir + "/tabs.skin", KConfig::SimpleConfig);
+    KConfig tabConfig(skinDir + QStringLiteral("/tabs.skin"), KConfig::SimpleConfig);
     KConfigGroup tabDescription = tabConfig.group("Description");
 
     titleName = titleDescription.readEntry("Skin", "");
@@ -178,7 +172,7 @@ QStandardItem* AppearanceSettings::createSkinItem(const QString& skinDir)
         skinName = skinId;
 
     if (skinAuthor.isEmpty())
-        skinAuthor = i18nc("@item:inlistbox Unknown skin author", "Unknown");
+        skinAuthor = xi18nc("@item:inlistbox Unknown skin author", "Unknown");
 
     QStandardItem* skin = new QStandardItem(skinName);
 
@@ -216,67 +210,77 @@ void AppearanceSettings::resetSelection()
 
 void AppearanceSettings::installSkin()
 {
-    QString mimeFilter = "application/x-tar application/x-compressed-tar "
-                         "application/x-bzip-compressed-tar application/zip";
+    QStringList mimeTypes;
+    mimeTypes << QStringLiteral("application/x-compressed-tar");
+    mimeTypes << QStringLiteral("application/x-xz-compressed-tar");
+    mimeTypes << QStringLiteral("application/x-bzip-compressed-tar");
+    mimeTypes << QStringLiteral("application/zip");
+    mimeTypes << QStringLiteral("application/x-tar");
 
-    KUrl skinUrl = KFileDialog::getOpenUrl(KUrl(), mimeFilter, parentWidget());
-
-    if (skinUrl.isEmpty()) return;
-
-    if (!KIO::NetAccess::download(skinUrl, m_installSkinFile, KApplication::activeWindow()))
-    {
-        KMessageBox::error(parentWidget(), KIO::NetAccess::lastErrorString(),
-            i18nc("@title:window", "Failed to Download Skin"));
+    QFileDialog fileDialog(parentWidget());
+    fileDialog.setWindowTitle(i18nc("@title:window", "Select the skin archive to install"));
+    fileDialog.setMimeTypeFilters(mimeTypes);
+    fileDialog.setFileMode(QFileDialog::ExistingFile);
 
+    QUrl skinUrl;
+    if (fileDialog.exec() && !fileDialog.selectedUrls().isEmpty())
+        skinUrl = fileDialog.selectedUrls().at(0);
+    else
         return;
-    }
 
-    QDir skinDir(m_installSkinFile);
+    m_installSkinFile.open();
 
-    if (!skinDir.exists())
-    {
-        KIO::ListJob* job = KIO::listRecursive("tar:" + m_installSkinFile, KIO::HideProgressInfo, false);
+    KIO::CopyJob *job = KIO::copy(skinUrl, QUrl::fromLocalFile(m_installSkinFile.fileName()), KIO::JobFlag::HideProgressInfo | KIO::JobFlag::Overwrite);
+    connect(job, &KIO::CopyJob::result, [=](KJob *job) {
+        if (job->error())
+        {
+            job->uiDelegate()->showErrorMessage();
+            job->kill();
 
-        connect(job, SIGNAL(entries(KIO::Job*,KIO::UDSEntryList)),
-            this, SLOT(listSkinArchive(KIO::Job*,KIO::UDSEntryList)));
+            cleanupAfterInstall();
 
-        connect(job, SIGNAL(result(KJob*)), this, SLOT(validateSkinArchive(KJob*)));
-    }
-    else
-        failInstall(i18nc("@info", "The installer was given a directory, not a file."));
+            return;
+        }
+
+        installSkin(static_cast<KIO::CopyJob*>(job)->destUrl());
+    });
 }
 
-void AppearanceSettings::listSkinArchive(KIO::Job* /* job */, const KIO::UDSEntryList& list)
+void AppearanceSettings::installSkin(const QUrl& skinUrl)
 {
-    if (list.count() == 0) return;
+    QUrl skinArchiveUrl = QUrl(skinUrl);
+    skinArchiveUrl.setScheme(QStringLiteral("tar"));
 
-    QListIterator<KIO::UDSEntry> i(list);
+    KIO::ListJob* job = KIO::listRecursive(skinArchiveUrl, KIO::HideProgressInfo, false);
+    connect(job, &KIO::ListJob::entries, [=](KIO::Job* /* job */, const KIO::UDSEntryList& list) {
+        if (list.count() == 0) return;
 
-    while (i.hasNext())
-        m_installSkinFileList.append(i.next().stringValue(KIO::UDSEntry::UDS_NAME));
-}
+        QListIterator<KIO::UDSEntry> i(list);
+        while (i.hasNext())
+            m_installSkinFileList.append(i.next().stringValue(KIO::UDSEntry::UDS_NAME));
+    });
 
-void AppearanceSettings::validateSkinArchive(KJob* job)
-{
-    if (!job->error())
-    {
-        m_installSkinId = m_installSkinFileList.at(0);
+    connect(job, &KIO::ListJob::result, [=](KJob *job) {
+        if (!job->error())
+        {
+            m_installSkinId = m_installSkinFileList.at(0);
 
-        if (validateSkin(m_installSkinId, m_installSkinFileList))
-            checkForExistingSkin();
+            if (validateSkin(m_installSkinId, m_installSkinFileList))
+                checkForExistingSkin();
+            else
+                failInstall(xi18nc("@info", "Unable to locate required files in the skin archive.<nl/><nl/>The archive appears to be invalid."));
+        }
         else
-            failInstall(i18nc("@info", "Unable to locate required files in the skin archive.<nl/><nl/>The archive appears to be invalid."));
-    }
-    else
-        failInstall(i18nc("@info", "Unable to list the skin archive contents.") + "\n\n" + job->errorString());
+            failInstall(xi18nc("@info", "Unable to list the skin archive contents.") + QStringLiteral("\n\n") + job->errorString());
+    });
 }
 
 bool AppearanceSettings::validateSkin(const QString &skinId, const QStringList& fileList)
 {
     bool titleFileFound = false;
     bool tabsFileFound = false;
-    QString titleFileName = skinId + "/title.skin";
-    QString tabsFileName = skinId + "/tabs.skin";
+    QString titleFileName = skinId + QStringLiteral("/title.skin");
+    QString tabsFileName = skinId + QStringLiteral("/tabs.skin");
 
     foreach (const QString& fileName, fileList)
     {
@@ -309,25 +313,24 @@ void AppearanceSettings::checkForExistingSkin()
     if (exists > 0)
     {
         QString skinDir = skins.at(0).data(SkinDir).toString();
-        QFile skin(skinDir + "/titles.skin");
+        QFile skin(skinDir + QStringLiteral("titles.skin"));
 
         if (!skin.open(QIODevice::ReadWrite))
         {
-            failInstall(i18nc("@info", "This skin appears to be already installed and you lack the required permissions to overwrite it."));
+            failInstall(xi18nc("@info", "This skin appears to be already installed and you lack the required permissions to overwrite it."));
         }
         else
         {
             skin.close();
 
             int remove = KMessageBox::warningContinueCancel(parentWidget(),
-                i18nc("@info", "This skin appears to be already installed. Do you want to overwrite it?"),
-                i18nc("@title:window", "Skin Already Exists"),
-                KGuiItem(i18nc("@action:button", "Reinstall Skin")));
+                xi18nc("@info", "This skin appears to be already installed. Do you want to overwrite it?"),
+                xi18nc("@title:window", "Skin Already Exists"),
+                KGuiItem(xi18nc("@action:button", "Reinstall Skin")));
 
             if (remove == KMessageBox::Continue)
             {
-                unlink(QFile::encodeName(skinDir));
-                KIO::DeleteJob* job = KIO::del(KUrl(skinDir), KIO::HideProgressInfo);
+                KIO::DeleteJob* job = KIO::del(QUrl::fromLocalFile(skinDir), KIO::HideProgressInfo);
                 connect(job, SIGNAL(result(KJob*)), this, SLOT(installSkinArchive(KJob*)));
             }
             else
@@ -342,12 +345,11 @@ void AppearanceSettings::installSkinArchive(KJob* deleteJob)
 {
     if (deleteJob && deleteJob->error())
     {
-        KMessageBox::error(parentWidget(), deleteJob->errorString(), i18nc("@title:Window", "Could Not Delete Skin"));
-
+        KMessageBox::error(parentWidget(), deleteJob->errorString(), xi18nc("@title:Window", "Could Not Delete Skin"));
         return;
     }
 
-    KTar skinArchive(m_installSkinFile);
+    KTar skinArchive(m_installSkinFile.fileName());
 
     if (skinArchive.open(QIODevice::ReadOnly))
     {
@@ -363,22 +365,26 @@ void AppearanceSettings::installSkinArchive(KJob* deleteJob)
         cleanupAfterInstall();
     }
     else
-        failInstall(i18nc("@info", "The skin archive file could not be opened."));
+        failInstall(xi18nc("@info", "The skin archive file could not be opened."));
 }
 
 void AppearanceSettings::failInstall(const QString& error)
 {
-    KMessageBox::error(parentWidget(), error, i18nc("@title:window", "Cannot Install Skin"));
+    KMessageBox::error(parentWidget(), error, xi18nc("@title:window", "Cannot Install Skin"));
 
     cleanupAfterInstall();
 }
 
 void AppearanceSettings::cleanupAfterInstall()
 {
-    KIO::NetAccess::removeTempFile(m_installSkinFile);
     m_installSkinId.clear();
-    m_installSkinFile.clear();
     m_installSkinFileList.clear();
+
+    if (m_installSkinFile.exists())
+    {
+        m_installSkinFile.close();
+        m_installSkinFile.remove();
+    }
 }
 
 void AppearanceSettings::updateRemoveSkinButton()
@@ -406,7 +412,7 @@ void AppearanceSettings::updateRemoveSkinButton()
         return;
     }
 
-    QFile titleSkin(skinDir + "/title.skin");
+    QFile titleSkin(skinDir + QStringLiteral("/title.skin"));
 
     if (!titleSkin.open(QIODevice::ReadWrite))
         removeButton->setEnabled(false);
@@ -428,31 +434,30 @@ void AppearanceSettings::removeSelectedSkin()
     if (skinDir.isEmpty()) return;
 
     int remove = KMessageBox::warningContinueCancel(parentWidget(),
-            i18nc("@info", "Do you want to remove \"%1\" by %2?", skinName, skinAuthor),
-            i18nc("@title:window", "Remove Skin"),
+            xi18nc("@info", "Do you want to remove \"%1\" by %2?", skinName, skinAuthor),
+            xi18nc("@title:window", "Remove Skin"),
             KStandardGuiItem::del());
 
     if (remove == KMessageBox::Continue)
     {
-        unlink(QFile::encodeName(skinDir));
-
-        bool deleted = KIO::NetAccess::del(KUrl(skinDir), KApplication::activeWindow());
-
-        if (deleted)
-        {
-            if (skinId == Settings::skin())
+        KIO::DeleteJob* job = KIO::del(QUrl::fromLocalFile(skinDir), KIO::HideProgressInfo);
+        connect(job, &KIO::DeleteJob::result, [=](KJob* deleteJob) {
+            if (deleteJob->error())
             {
-                Settings::setSkin("default");
-                Settings::setSkinInstalledWithKns(false);
-                Settings::self()->writeConfig();
-                emit settingsChanged();
-            }
+                KMessageBox::error(parentWidget(), deleteJob->errorString(), xi18nc("@title:Window", "Could Not Delete Skin"));
+            } else {
+                if (skinId == Settings::skin())
+                {
+                    Settings::setSkin(QStringLiteral("default"));
+                    Settings::setSkinInstalledWithKns(false);
+                    Settings::self()->save();
+                    emit settingsChanged();
+                }
 
-            resetSelection();
-            populateSkinList();
-        }
-        else
-            KMessageBox::error(parentWidget(), i18nc("@info", "Could not remove skin \"%1\".", skinName));
+                resetSelection();
+                populateSkinList();
+            }
+        });
     }
 }
 
@@ -469,13 +474,13 @@ QSet<QString> AppearanceSettings::extractKnsSkinIds(const QStringList& fileList)
             QString relativeName = QString(file).remove(m_knsSkinDir, Qt::CaseInsensitive);
 
             // Get everything before the first slash - that should be our skins ID.
-            QString skinId = relativeName.section('/', 0, QString::SectionSkipEmpty);
+            QString skinId = relativeName.section(QStringLiteral("/"), 0, QString::SectionSkipEmpty);
 
             // Skip all other entries in the file list if we found what we were searching for.
             if (!skinId.isEmpty())
             {
                 // First remove all remaining slashes (as there could be leading or trailing ones).
-                skinId = skinId.replace('/', QString());
+                skinId = skinId.replace(QStringLiteral("/"), QString());
 
                 skinIdList.insert(skinId);
             }
@@ -496,7 +501,6 @@ void AppearanceSettings::getNewSkins()
 
     if (dialog.isNull())
     {
-        kDebug() << "KNS3::DownloadDialog was destroyed during exec()!";
         return;
     }
 
@@ -505,8 +509,9 @@ void AppearanceSettings::getNewSkins()
         quint32 invalidEntryCount = 0;
         QString invalidSkinText;
 
-        foreach (const KNS3::Entry &entry, dialog->installedEntries())
+        foreach (const KNS3::Entry &entry3, dialog->installedEntries())
         {
+            KNSCore::EntryInternal entry = KNSCore::EntryInternal::fromEntry(entry3);
             bool isValid = true;
             const QSet<QString>& skinIdList = extractKnsSkinIds(entry.installedFiles());
 
@@ -516,8 +521,6 @@ void AppearanceSettings::getNewSkins()
                 // Validate the current skin.
                 if (!validateSkin(skinId, entry.installedFiles()))
                 {
-                    kDebug() << "skinId '" << skinId << "' is invalid "
-                             << "(it's either missing the 'title.skin' or 'tabs.skin' file).";
                     isValid = false;
                 }
             }
@@ -532,7 +535,7 @@ void AppearanceSettings::getNewSkins()
 
                 // The user needs to know the name of the skin which
                 // was removed.
-                invalidSkinText += QString("<li>%1</li>").arg(entry.name());
+                invalidSkinText += QString(QStringLiteral("<li>%1</li>")).arg(entry.name());
 
                 // Then remove the skin.
                 m_knsDownloadManager->uninstallEntry(entry);
@@ -542,7 +545,7 @@ void AppearanceSettings::getNewSkins()
         // Are there any invalid entries?
         if (invalidEntryCount > 0)
         {
-            failInstall(i18ncp("@info",
+            failInstall(xi18ncp("@info",
                                "The following skin is missing required files. Thus it was removed:<ul>%2</ul>",
                                "The following skins are missing required files. Thus they were removed:<ul>%2</ul>",
                                invalidEntryCount,
